import 'package:flutter/material.dart';
import 'package:flutter_custom_toolbox/flutter_toolbox.dart';

import 'package:jeweled/common/cubit/nav/nav_cubit_pages.dart';

import 'package:jeweled/cubit/activity/activity_cubit.dart';

import 'package:jeweled/ui/parameters/parameter_painter_board_size.dart';
import 'package:jeweled/ui/parameters/parameter_painter_colors_count.dart';
import 'package:jeweled/ui/parameters/parameter_painter_color_theme.dart';
import 'package:jeweled/ui/parameters/parameter_painter_graphic_theme.dart';

class ApplicationConfig {
  // known parameters
  static const String parameterCodeColorTheme = 'global.colorTheme';
  static const String parameterCodeGraphicsTheme = 'global.graphicTheme';
  static const String parameterCodeBoardSize = 'activity.boardSize';
  static const String parameterCodeColorsCount = 'activity.colorsCount';

  // colors theme values
  static const String colorThemeGothicBit = 'gothic-bit';
  static const String colorThemeSweethope = 'sweethope';
  static const String colorThemeNostalgicDreams = 'nostalgic-dreams';
  static const String colorThemeArjibi8 = 'arjibi8';

  // graphic theme values
  static const String graphicThemeSolidBackground = 'SolidBackground';
  static const String graphicThemeGradientAndBorder = 'GradientAndBorder';
  static const String graphicThemeEmojis = 'Emojis';
  static const String graphicThemePatterns = 'Patterns';

  // board size values
  static const String boardSizeValueSmall = '6';
  static const String boardSizeValueMedium = '10';
  static const String boardSizeValueLarge = '14';
  static const String boardSizeValueExtraLarge = '18';

  // colors count values
  static const String colorsCountValueLow = '5';
  static const String colorsCountValueMedium = '6';
  static const String colorsCountValueHigh = '7';
  static const String colorsCountValueVeryHigh = '8';

  static int blockMinimumCellsCount = 3;

  static final ApplicationConfigDefinition config = ApplicationConfigDefinition(
    appTitle: 'Jeweled',
    activitySettings: [
      // board size
      ApplicationSettingsParameter(
        code: parameterCodeBoardSize,
        values: [
          ApplicationSettingsParameterItemValue(
            value: boardSizeValueSmall,
            color: Colors.green,
          ),
          ApplicationSettingsParameterItemValue(
            value: boardSizeValueMedium,
            color: Colors.orange,
            isDefault: true,
          ),
          ApplicationSettingsParameterItemValue(
            value: boardSizeValueLarge,
            color: Colors.red,
          ),
          ApplicationSettingsParameterItemValue(
            value: boardSizeValueExtraLarge,
            color: Colors.purple,
          ),
        ],
        customPainter: (context, value) {
          return ParameterPainterBoardSize(value: value, context: context);
        },
      ),

      // colors count
      ApplicationSettingsParameter(
        code: parameterCodeColorsCount,
        values: [
          ApplicationSettingsParameterItemValue(
            value: colorsCountValueLow,
            color: Colors.green,
          ),
          ApplicationSettingsParameterItemValue(
            value: colorsCountValueMedium,
            color: Colors.orange,
            isDefault: true,
          ),
          ApplicationSettingsParameterItemValue(
            value: colorsCountValueHigh,
            color: Colors.red,
          ),
          ApplicationSettingsParameterItemValue(
            value: colorsCountValueVeryHigh,
            color: Colors.purple,
          ),
        ],
        customPainter: (context, value) {
          return ParameterPainterColorsCount(value: value, context: context);
        },
      ),

      // colors theme
      ApplicationSettingsParameter(
        code: parameterCodeColorTheme,
        displayedOnTop: false,
        values: [
          ApplicationSettingsParameterItemValue(
            value: colorThemeGothicBit,
          ),
          ApplicationSettingsParameterItemValue(
            value: colorThemeSweethope,
            isDefault: true,
          ),
          ApplicationSettingsParameterItemValue(
            value: colorThemeNostalgicDreams,
          ),
          ApplicationSettingsParameterItemValue(
            value: colorThemeArjibi8,
          ),
        ],
        customPainter: (context, value) {
          return ParameterPainterColorTheme(value: value, context: context);
        },
      ),

      // graphics theme
      ApplicationSettingsParameter(
        code: parameterCodeGraphicsTheme,
        displayedOnTop: false,
        values: [
          ApplicationSettingsParameterItemValue(
            value: graphicThemeSolidBackground,
          ),
          ApplicationSettingsParameterItemValue(
            value: graphicThemeGradientAndBorder,
            isDefault: true,
          ),
          ApplicationSettingsParameterItemValue(
            value: graphicThemeEmojis,
          ),
          ApplicationSettingsParameterItemValue(
            value: graphicThemePatterns,
          ),
        ],
        customPainter: (context, value) {
          return ParameterPainterGraphicTheme(value: value, context: context);
        },
      ),
    ],
    startNewActivity: (BuildContext context) {
      BlocProvider.of<ActivityCubit>(context).startNewActivity(context);
      BlocProvider.of<NavCubitPage>(context).goToPageGame();
    },
    deleteCurrentActivity: (BuildContext context) {
      BlocProvider.of<ActivityCubit>(context).deleteSavedActivity();
    },
    resumeActivity: (BuildContext context) {
      BlocProvider.of<ActivityCubit>(context).resumeSavedActivity();
      BlocProvider.of<NavCubitPage>(context).goToPageGame();
    },
  );

  static const List<String> graphicThemeContentEmojiStrings = [
    '🍏',
    '🤍',
    '🦋',
    '🐞',
    '⭐',
    '🍄',
    '🍒',
    '🐤',
  ];
  static const List<String> graphicThemeContentPatternStrings = [
    '✖',
    '✚',
    '▲',
    '■',
    '●',
    '◆',
    '━',
    '⧧',
  ];
}
